use std::fmt::Write;

pub struct IndentWriter {
    indent_level: usize,
    indent_string: String,
    out: Box<dyn Write>,
}

impl IndentWriter {
    pub fn new(start_level: usize, indent_string: String, out: Box<dyn Write>) -> IndentWriter {
        IndentWriter {
            indent_level: start_level,
            indent_string,
            out,
        }
    }

    pub fn increase_indent(&mut self) {
        self.indent_level += 1;
    }

    pub fn decrease_indent(&mut self) {
        self.indent_level -= 1;
    }

    pub fn write(&mut self, s: &str) -> std::fmt::Result {
        write!(self.out, "{}", s)
    }
    
    pub fn writeln(&mut self, s: &str) -> std::fmt::Result {
        self.write(&format!("{}\n", s))
    }

    pub fn write_indented(&mut self, s: &str) -> std::fmt::Result {
        for _ in 0..self.indent_level {
            write!(self.out, "{}", self.indent_string)?;
        }
        write!(self.out, "{}", s)
    }
    
    pub fn writeln_indented(&mut self, s: &str) -> std::fmt::Result {
        self.write_indented(&format!("{}\n", s))
    }
}
